@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -123,6 +115,8 @@ protected:
     virtual void updateDirection(QAbstractAnimationJob::Direction direction);
     virtual void topLevelAnimationLoopChanged() {}
 
+    virtual void debugAnimation(QDebug d) const;
+
     void fireTopLevelAnimationLoopChanged();
 
     void setState(QAbstractAnimationJob::State state);
@@ -169,6 +163,7 @@ protected:
 
     friend class QQmlAnimationTimer;
     friend class QAnimationGroupJob;
+    friend QDebug operator<<(QDebug, const QAbstractAnimationJob *job);
 };
 
 class Q_QML_PRIVATE_EXPORT QAnimationJobChangeListener
@@ -209,18 +204,17 @@ public:
     void restartAnimationTimer();
     void updateAnimationsTime(qint64 timeStep);
 
-    int currentDelta() { return lastDelta; }
-
     //useful for profiling/debugging
     int runningAnimationCount() { return animations.count(); }
 
-private Q_SLOTS:
+    bool hasStartAnimationPending() const { return startAnimationPending; }
+
+public Q_SLOTS:
     void startAnimations();
     void stopTimer();
 
 private:
     qint64 lastTick;
-    int lastDelta;
     int currentAnimationIdx;
     bool insideTick;
     bool startAnimationPending;
@@ -240,6 +234,8 @@ private:
 
 Q_DECLARE_OPERATORS_FOR_FLAGS(QAbstractAnimationJob::ChangeTypes)
 
+QDebug operator<<(QDebug, const QAbstractAnimationJob *job);
+
 QT_END_NAMESPACE
 
 #endif // QABSTRACTANIMATIONJOB_P_H
