@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtQml module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -106,6 +98,7 @@ struct QQmlMetaTypeData
     QBitArray lists;
 
     QList<QQmlPrivate::AutoParentFunction> parentFunctions;
+    QVector<QQmlPrivate::QmlUnitCacheLookupFunction> lookupCachedQmlUnit;
 
     QSet<QString> protectedNamespaces;
 
@@ -116,7 +109,7 @@ struct QQmlMetaTypeData
 class QQmlTypeModulePrivate
 {
 public:
-    QQmlTypeModulePrivate() 
+    QQmlTypeModulePrivate()
     : minMinorVersion(INT_MAX), maxMinorVersion(0), locked(false) {}
 
     static QQmlTypeModulePrivate* get(QQmlTypeModule* q) { return q->d; }
@@ -231,7 +224,10 @@ void QQmlType::SingletonInstanceInfo::init(QQmlEngine *e)
         v4->popContext();
     } else if (qobjectCallback && !qobjectApi(e)) {
         v4->pushGlobalContext();
-        setQObjectApi(e, qobjectCallback(e, e));
+        QObject *o = qobjectCallback(e, e);
+        setQObjectApi(e, o);
+        // if this object can use a property cache, create it now
+        QQmlData::ensurePropertyCache(e, o);
         v4->popContext();
     } else if (!url.isEmpty() && !qobjectApi(e)) {
         v4->pushGlobalContext();
@@ -479,7 +475,7 @@ QQmlType *QQmlType::superType() const
     return d->superType;
 }
 
-static void clone(QMetaObjectBuilder &builder, const QMetaObject *mo, 
+static void clone(QMetaObjectBuilder &builder, const QMetaObject *mo,
                   const QMetaObject *ignoreStart, const QMetaObject *ignoreEnd)
 {
     // Set classname
@@ -491,7 +487,7 @@ static void clone(QMetaObjectBuilder &builder, const QMetaObject *mo,
 
         int otherIndex = ignoreEnd->indexOfClassInfo(info.name());
         if (otherIndex >= ignoreStart->classInfoOffset() + ignoreStart->classInfoCount()) {
-            // Skip 
+            // Skip
         } else {
             builder.addClassInfo(info.name(), info.value());
         }
@@ -504,7 +500,7 @@ static void clone(QMetaObjectBuilder &builder, const QMetaObject *mo,
         int otherIndex = ignoreEnd->indexOfProperty(property.name());
         if (otherIndex >= ignoreStart->propertyOffset() + ignoreStart->propertyCount()) {
             builder.addProperty(QByteArray("__qml_ignore__") + property.name(), QByteArray("void"));
-            // Skip 
+            // Skip
         } else {
             builder.addProperty(property);
         }
@@ -520,7 +516,7 @@ static void clone(QMetaObjectBuilder &builder, const QMetaObject *mo,
 
         bool found = false;
 
-        for (int ii = ignoreStart->methodOffset() + ignoreStart->methodCount(); 
+        for (int ii = ignoreStart->methodOffset() + ignoreStart->methodCount();
              !found && ii < ignoreEnd->methodOffset() + ignoreEnd->methodCount();
              ++ii) {
 
@@ -540,7 +536,7 @@ static void clone(QMetaObjectBuilder &builder, const QMetaObject *mo,
 
         int otherIndex = ignoreEnd->indexOfEnumerator(enumerator.name());
         if (otherIndex >= ignoreStart->enumeratorOffset() + ignoreStart->enumeratorCount()) {
-            // Skip 
+            // Skip
         } else {
             builder.addEnumerator(enumerator);
         }
@@ -664,7 +660,7 @@ void QQmlTypePrivate::insertEnums(const QMetaObject *metaObject) const
 {
     // Add any enum values defined by 'related' classes
     if (metaObject->d.relatedMetaObjects) {
-        const QMetaObject **related = metaObject->d.relatedMetaObjects;
+        const QMetaObject * const *related = metaObject->d.relatedMetaObjects;
         if (related) {
             while (*related)
                 insertEnums(*related++);
@@ -860,7 +856,7 @@ const QMetaObject *QQmlType::attachedPropertiesType() const
 
 /*
 This is the id passed to qmlAttachedPropertiesById().  This is different from the index
-for the case that a single class is registered under two or more names (eg. Item in 
+for the case that a single class is registered under two or more names (eg. Item in
 Qt 4.7 and QtQuick 1.0).
 */
 int QQmlType::attachedPropertiesId() const
@@ -1128,7 +1124,7 @@ int registerAutoParentFunction(QQmlPrivate::RegisterAutoParent &autoparent)
 
 int registerInterface(const QQmlPrivate::RegisterInterface &interface)
 {
-    if (interface.version > 0) 
+    if (interface.version > 0)
         qFatal("qmlRegisterType(): Cannot mix incompatible QML versions.");
 
     QWriteLocker lock(metaTypeDataLock());
@@ -1344,6 +1340,15 @@ int registerCompositeType(const QQmlPrivate::RegisterCompositeType &type)
     return index;
 }
 
+int registerQmlUnitCacheHook(const QQmlPrivate::RegisterQmlUnitCacheHook &hookRegistration)
+{
+    if (hookRegistration.version > 0)
+        qFatal("qmlRegisterType(): Cannot mix incompatible QML versions.");
+    QWriteLocker lock(metaTypeDataLock());
+    QQmlMetaTypeData *data = metaTypeData();
+    data->lookupCachedQmlUnit << hookRegistration.lookupCachedQmlUnit;
+    return 0;
+}
 
 /*
 This method is "over generalized" to allow us to (potentially) register more types of things in
@@ -1363,6 +1368,8 @@ int QQmlPrivate::qmlregister(RegistrationType type, void *data)
         return registerCompositeType(*reinterpret_cast<RegisterCompositeType *>(data));
     } else if (type == CompositeSingletonRegistration) {
         return registerCompositeSingletonType(*reinterpret_cast<RegisterCompositeSingletonType *>(data));
+    } else if (type == QmlUnitCacheHookRegistration) {
+        return registerQmlUnitCacheHook(*reinterpret_cast<RegisterQmlUnitCacheHook *>(data));
     }
     return -1;
 }
@@ -1472,7 +1479,7 @@ bool QQmlMetaType::isModule(const QString &module, int versionMajor, int version
     QQmlMetaTypeData *data = metaTypeData();
 
     // first, check Types
-    QQmlTypeModule *tm = 
+    QQmlTypeModule *tm =
         data->uriToModule.value(QQmlMetaTypeData::VersionedUri(module, versionMajor));
     if (tm && tm->minimumMinorVersion() <= versionMinor && tm->maximumMinorVersion() >= versionMinor)
         return true;
@@ -1755,7 +1762,7 @@ QQmlType *QQmlMetaType::qmlType(const QMetaObject *metaObject, const QHashedStri
 }
 
 /*!
-    Returns the type (if any) that corresponds to the QVariant::Type \a userType.  
+    Returns the type (if any) that corresponds to the QVariant::Type \a userType.
     Returns null if no type is registered.
 */
 QQmlType *QQmlMetaType::qmlType(int userType)
@@ -1835,6 +1842,17 @@ QList<QQmlType*> QQmlMetaType::qmlTypes()
     return data->nameToType.values();
 }
 
+/*!
+    Returns the list of all registered types.
+*/
+QList<QQmlType*> QQmlMetaType::qmlAllTypes()
+{
+    QReadLocker lock(metaTypeDataLock());
+    QQmlMetaTypeData *data = metaTypeData();
+
+    return data->types;
+}
+
 /*!
     Returns the list of registered QML singleton types.
 */
@@ -1854,26 +1872,16 @@ QList<QQmlType*> QQmlMetaType::qmlSingletonTypes()
     return retn;
 }
 
-int QQmlMetaType::QQuickAnchorLineMetaTypeId()
+const QQmlPrivate::CachedQmlUnit *QQmlMetaType::findCachedCompilationUnit(const QUrl &uri)
 {
-    static int id = 0;
-    if (!id) {
-        id = QMetaType::type("QQuickAnchorLine");
+    QReadLocker lock(metaTypeDataLock());
+    QQmlMetaTypeData *data = metaTypeData();
+    for (QVector<QQmlPrivate::QmlUnitCacheLookupFunction>::ConstIterator it = data->lookupCachedQmlUnit.constBegin(), end = data->lookupCachedQmlUnit.constEnd();
+         it != end; ++it) {
+        if (const QQmlPrivate::CachedQmlUnit *unit = (*it)(uri))
+            return unit;
     }
-    return id;
-}
-
-QQmlMetaType::CompareFunction QQmlMetaType::anchorLineCompareFunction = 0;
-
-void QQmlMetaType::setQQuickAnchorLineCompareFunction(CompareFunction fun)
-{
-    anchorLineCompareFunction = fun;
-}
-
-bool QQmlMetaType::QQuickAnchorLineCompare(const void *p1, const void *p2)
-{
-    Q_ASSERT(anchorLineCompareFunction != 0);
-    return anchorLineCompareFunction(p1, p2);
+    return 0;
 }
 
 QT_END_NAMESPACE
