@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the QtQuick module of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -53,8 +45,6 @@
 #include <private/qquickpixmapcache_p.h>
 #include <private/qtquickglobal_p.h>
 
-QT_BEGIN_HEADER
-
 QT_BEGIN_NAMESPACE
 
 class QQuickSprite;
@@ -110,12 +100,12 @@ public:
         return m_randomStart;
     }
 
-signals:
+Q_SIGNALS:
     void durationChanged(int arg);
 
-    void nameChanged(QString arg);
+    void nameChanged(const QString &arg);
 
-    void toChanged(QVariantMap arg);
+    void toChanged(const QVariantMap &arg);
 
     void durationVariationChanged(int arg);
 
@@ -123,28 +113,28 @@ signals:
 
     void randomStartChanged(bool arg);
 
-public slots:
+public Q_SLOTS:
     void setDuration(int arg)
     {
         if (m_duration != arg) {
             m_duration = arg;
-            emit durationChanged(arg);
+            Q_EMIT durationChanged(arg);
         }
     }
 
-    void setName(QString arg)
+    void setName(const QString &arg)
     {
         if (m_name != arg) {
             m_name = arg;
-            emit nameChanged(arg);
+            Q_EMIT nameChanged(arg);
         }
     }
 
-    void setTo(QVariantMap arg)
+    void setTo(const QVariantMap &arg)
     {
         if (m_to != arg) {
             m_to = arg;
-            emit toChanged(arg);
+            Q_EMIT toChanged(arg);
         }
     }
 
@@ -152,7 +142,7 @@ public slots:
     {
         if (m_durationVariation != arg) {
             m_durationVariation = arg;
-            emit durationVariationChanged(arg);
+            Q_EMIT durationVariationChanged(arg);
         }
     }
 
@@ -160,7 +150,7 @@ public slots:
     {
         if (m_randomStart != arg) {
             m_randomStart = arg;
-            emit randomStartChanged(arg);
+            Q_EMIT randomStartChanged(arg);
         }
     }
 
@@ -216,17 +206,17 @@ public:
 
     int stateCount() {return m_states.count();}
 private:
-signals:
+Q_SIGNALS:
 
-    void globalGoalChanged(QString arg);
+    void globalGoalChanged(const QString &arg);
     void stateChanged(int idx);
 
-public slots:
-    void setGlobalGoal(QString arg)
+public Q_SLOTS:
+    void setGlobalGoal(const QString &arg)
     {
         if (m_globalGoal != arg) {
             m_globalGoal = arg;
-            emit globalGoalChanged(arg);
+            Q_EMIT globalGoalChanged(arg);
         }
     }
 
@@ -323,6 +313,4 @@ inline int spriteCount(QQmlListProperty<QQuickSprite> *p)
 
 QT_END_NAMESPACE
 
-QT_END_HEADER
-
 #endif // QQUICKSPRITEENGINE_P_H
