@@ -1,40 +1,32 @@
 /****************************************************************************
 **
-** Copyright (C) 2013 Digia Plc and/or its subsidiary(-ies).
+** Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 ** Contact: http://www.qt-project.org/legal
 **
 ** This file is part of the test suite of the Qt Toolkit.
 **
-** $QT_BEGIN_LICENSE:LGPL$
+** $QT_BEGIN_LICENSE:LGPL21$
 ** Commercial License Usage
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
+** a written agreement between you and Digia. For licensing terms and
+** conditions see http://qt.digia.com/licensing. For further information
 ** use the contact form at http://qt.digia.com/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
 ** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** rights. These rights are described in the Digia Qt LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3.0 as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU General Public License version 3.0 requirements will be
-** met: http://www.gnu.org/copyleft/gpl.html.
-**
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -43,16 +35,61 @@
 
 #include <QtQuick>
 
-#include <private/qsgcontext_p.h>
-
+#include <private/qopenglcontext_p.h>
 
 #include <QtQml>
 
+class PerPixelRect : public QQuickItem
+{
+    Q_PROPERTY(QColor color READ color WRITE setColor NOTIFY colorChanged)
+    Q_OBJECT
+public:
+    PerPixelRect() {
+        setFlag(ItemHasContents);
+    }
+
+    void setColor(const QColor &c) {
+        if (c == m_color)
+            return;
+        m_color = c;
+        emit colorChanged(c);
+    }
+
+    QColor color() const { return m_color; }
+
+    QSGNode *updatePaintNode(QSGNode *old, UpdatePaintNodeData *)
+    {
+        if (old)
+            delete old;
+
+        QSGNode *node = new QSGNode();
+
+        for (int y=0; y<height(); ++y) {
+            for (int x=0; x<width(); ++x) {
+                QSGSimpleRectNode *rn = new QSGSimpleRectNode();
+                rn->setRect(x, y, 1, 1);
+                rn->setColor(m_color);
+                node->appendChildNode(rn);
+            }
+        }
+
+        return node;
+    }
+
+Q_SIGNALS:
+    void colorChanged(const QColor &c   );
+
+private:
+    QColor m_color;
+};
+
 class tst_SceneGraph : public QObject
 {
     Q_OBJECT
 
 private slots:
+    void initTestCase();
+
     void manyWindows_data();
     void manyWindows();
 
@@ -67,6 +104,11 @@ public:
     ~ScopedList() { qDeleteAll(*this); }
 };
 
+void tst_SceneGraph::initTestCase()
+{
+    qmlRegisterType<PerPixelRect>("SceneGraphTest", 1, 0, "PerPixelRect");
+}
+
 QQuickView *createView(const QString &file, QWindow *parent = 0, int x = -1, int y = -1, int w = -1, int h = -1)
 {
     QQuickView *view = new QQuickView(parent);
@@ -168,7 +210,7 @@ void tst_SceneGraph::manyWindows_data()
 
 struct ShareContextResetter {
 public:
-    ~ShareContextResetter() { QSGContext::setSharedOpenGLContext(0); }
+    ~ShareContextResetter() { QOpenGLContextPrivate::setGlobalShareContext(0); }
 };
 
 void tst_SceneGraph::manyWindows()
@@ -181,7 +223,7 @@ void tst_SceneGraph::manyWindows()
     ShareContextResetter cleanup; // To avoid dangling pointer in case of test-failure.
     if (shared) {
         sharedGLContext.create();
-        QSGContext::setSharedOpenGLContext(&sharedGLContext);
+        QOpenGLContextPrivate::setGlobalShareContext(&sharedGLContext);
     }
 
     QScopedPointer<QWindow> parent;
@@ -328,7 +370,8 @@ void tst_SceneGraph::render_data()
           << "data/render_StackingOrder.qml"
           << "data/render_Mipmap.qml"
           << "data/render_ImageFiltering.qml"
-          << "data/render_bug37555.qml"
+          << "data/render_bug37422.qml"
+          << "data/render_OpacityThroughBatchRoot.qml"
         ;
 
     QRegExp sampleCount("#samples: *(\\d+)");
